"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var http_1 = require("http");
var os_1 = require("os");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var serveStatic = require("serve-static");
var uuid_1 = require("uuid");
var add_library_command_1 = require("../src/commands/add.library.command");
var rx_copy_1 = require("../src/utils/rx.copy");
var rx_delete_1 = require("../src/utils/rx.delete");
var expect_utils_1 = require("./utils/expect.utils");
var test_utils_1 = require("./utils/test.utils");
describe('add.library.command', function () {
    it('should import the text module', function () {
        // ramp up dev server
        var app = express();
        app.use(serveStatic(path_1.join(test_utils_1.getAssetBaseDir(), 'test', 'npm-registry'), { index: 'index.json' }));
        var server = http_1.createServer(app);
        var startServer = rxjs_1.bindNodeCallback(server.listen.bind(server));
        var rxServer = startServer(0).pipe(operators_1.mapTo(server), operators_1.shareReplay());
        var rxPort = rxServer.pipe(operators_1.map(function (server) { return server.address().port; }));
        var rxCloseServer = rxjs_1.defer(function () { return rxjs_1.bindNodeCallback(server.close.bind(server))(); });
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var dstSrc = dstBase;
        var assetBase = test_utils_1.getAssetBaseDir();
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        // copy the data
        var rxData = rx_copy_1.rxCopyDir(srcSrc, dstSrc);
        var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
        // create layouts on the result
        var rxCmdOpt = rxPort.pipe(operators_1.map(function (port) { return ({
            command: add_library_command_1.AddLibraryCommand.COMMAND,
            commandType: add_library_command_1.AddLibraryCommand.TYPE,
            pkgDir: dstSrc,
            url: "http://localhost:" + port + "/",
            name: '@dx-samples/text-and-image-component@^0.2.0',
            verbose: true,
            override: true,
            allArgs: []
        }); }));
        var rxLibCmd = rxCmdOpt.pipe(operators_1.map(function (opt) { return new add_library_command_1.AddLibraryCommand(opt); }));
        var rxCmd = rxLibCmd.pipe(operators_1.mergeMap(function (cmd) { return cmd.process(); }));
        var rxPrepareData = rxjs_1.concat(rxDelete, rxData);
        // setup teardown
        var rxSetup = rxjs_1.forkJoin(rxServer, rxPrepareData).pipe(operators_1.shareReplay());
        var rxTeardown = rxjs_1.forkJoin(rxCloseServer, rxDelete);
        // command
        var rxTest = rxCmd.pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray(), operators_1.map(function (files) {
            // make sure these files exist
            expect(test_utils_1.findFile('package.json', files)).toBeDefined();
            expect(test_utils_1.findFile('app.module.ts', files)).toBeDefined();
        }));
        // execute the command
        var rxTotal = rxjs_1.concat(rxSetup, rxTest, rxTeardown);
        return rxTotal.toPromise();
    });
});
