"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var Promise = require("bluebird");
var path_1 = require("path");
var RelateUrl = require("relateurl");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var url_1 = require("url");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var config_1 = require("./../utils/config");
var url_utils_1 = require("./../utils/url.utils");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
var cli = require('@angular/cli').default;
function _buildProduction(aAppName, aContextRoot, aApiBaseUrl, aSrcDir, aRelative, aSkipSourcemaps, aLogger) {
    // the result
    return rxjs_1.Observable.create(function (files) {
        // the delivery URL
        var absWchDeliveryUrl = url_utils_1.ensureTrailingSlash(wchtools_1.wchDeliveryUrlFromApiUrl(aApiBaseUrl));
        // path of the index file
        var absRootUrl = url_utils_1.ensureTrailingSlash("" + absWchDeliveryUrl + config_1.configTrimSlashes(aContextRoot));
        // get the base URL
        var absAppUrl = url_utils_1.ensureTrailingSlash(config_1.configGetBaseUrl(aAppName, aApiBaseUrl));
        // dist directory
        var distDir = path_1.join(aSrcDir, 'dist');
        // the base href
        var baseUrl;
        // check if we want a relative URL
        if (aRelative) {
            // parse
            var parsedUrl = url_1.parse(absRootUrl);
            baseUrl = parsedUrl.path || '/';
        }
        else {
            // use the absolute URL
            baseUrl = absRootUrl;
        }
        // the relative delivery URL
        var resolve = new RelateUrl(absRootUrl);
        var deliveryUrl = resolve.relate(absAppUrl);
        // log this
        aLogger.next("BaseUrl [" + baseUrl + "], deliveryUrl [" + deliveryUrl + "], source directory [" + aSrcDir + "].");
        // command line interface
        var args = ['build', '--prod', '--aot', '--build-optimizer', '--base-href', baseUrl];
        if (deliveryUrl && (deliveryUrl.length > 0)) {
            args.push('--deploy-url', deliveryUrl);
        }
        if (!aSkipSourcemaps) {
            args.push('--source-map');
        }
        aLogger.next("ng command line arguments " + JSON.stringify(args) + ".");
        // execute the command, we will sync on files
        Promise.resolve(cli({
            cliArgs: args,
            directory: aSrcDir,
            inputStream: process.stdin,
            outputStream: process.stdout
        }))
            .then(function () { return files.complete(); })
            .catch(function (err) { return files.error(err); });
    });
}
var BuildProductionCommand = /** @class */ (function (_super) {
    __extends(BuildProductionCommand, _super);
    function BuildProductionCommand(options) {
        return _super.call(this, options) || this;
    }
    BuildProductionCommand.prototype.printHelp = function () {
        // log the command line
        console.log(BuildProductionCommand.COMMAND, BuildProductionCommand.TYPE, "[--" + constants_1.OPTION_SOURCE + " <SOURCE>]", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_DATA + " <DATA>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_USER + " <USER>]", "[--" + constants_1.OPTION_PASSWORD + " <PASSWORD>]", "[--" + constants_1.OPTION_SKIP_SOURCEMAPS + "]", "[--" + constants_1.OPTION_RELATIVE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // help on the options
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_BUILD_SOURCE);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_USER + " <USER>", ':', help_1.HELP_USERNAME);
        console.log("--" + constants_1.OPTION_PASSWORD + " <PASSWORD>", ':', help_1.HELP_PASSWORD);
        console.log("--" + constants_1.OPTION_SKIP_SOURCEMAPS, ':', help_1.HELP_SKIP_SOURCEMAP);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
        console.log("--" + constants_1.OPTION_RELATIVE, ':', help_1.HELP_RELATIVE);
    };
    BuildProductionCommand.prototype.process = function () {
        // the options
        var opts = this.getOptions();
        // get the tools
        var apiBaseUrl = this.getApiBase();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // source directory
        var srcDir = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || this.getPackageDir());
        // relative flag
        var bRelative = !!opts.relative;
        // skip sourcemaps
        var bSkipSourcemaps = !!opts.skipSourcemaps;
        // logger
        var logger = this.getLogger();
        // log the config
        logger.next("AppName [" + appName + "], contextRoot [" + contextRoot + "]");
        // process
        return apiBaseUrl.pipe(operators_1.mergeMap(function (aApiBaseUrl) { return _buildProduction(appName, contextRoot, aApiBaseUrl, srcDir, bRelative, bSkipSourcemaps, logger); }));
    };
    BuildProductionCommand.TYPE = 'production';
    BuildProductionCommand.COMMAND = constants_1.CMD_BUILD;
    return BuildProductionCommand;
}(abstract_command_1.AbstractCommand));
exports.BuildProductionCommand = BuildProductionCommand;
