"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var escapeStringRegexp = require("escape-string-regexp");
var graceful_fs_1 = require("graceful-fs");
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var component_reg_1 = require("../utils/component.reg");
var file_utils_1 = require("../utils/file.utils");
var layout_reg_1 = require("../utils/layout.reg");
var rx_copy_1 = require("../utils/rx.copy");
var rx_utils_1 = require("../utils/rx.utils");
var type_reg_1 = require("../utils/type.reg");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var file_utils_2 = require("./../utils/file.utils");
var json_1 = require("./../utils/json");
var rx_file_1 = require("./../utils/rx.file");
var rx_hbs_1 = require("./../utils/rx.hbs");
var rx_lines_1 = require("./../utils/rx.lines");
var rx_tidy_1 = require("./../utils/rx.tidy");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var ts_add_layout_1 = require("./../utils/ts.add.layout");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
var ASSETS_BASE = path_1.join(__dirname, '..', '..', '..', 'assets');
var TYPE_GROUP = 'Group';
var TYPE_BINDING = 'RenderingContextBinding';
var TYPE_CONTEXT = 'RenderingContext';
var TYPE_ABSTRACT_COMPONENT = 'AbstractRenderingComponent';
var DEP_SDK = '@ibm-wch-sdk/ng';
var DEP_UTILS = '@ibm-wch-sdk/utils';
function _camelCase(aValue) {
    return transliteration_1.camelCase(aValue);
}
function _plural(aKey) {
    return aKey + 's';
}
function _toComment(aValue) {
    return JSON.stringify(json_1.canonicalizeJSON(aValue), undefined, 2)
        .split('\n')
        .map(function (line) { return '     * ' + line; })
        .join('\n');
}
var SINGLE_ELEMENT_TYPES = {};
SINGLE_ELEMENT_TYPES['text'] = 'SingleTextElement';
SINGLE_ELEMENT_TYPES['number'] = 'SingleNumberElement';
SINGLE_ELEMENT_TYPES['toggle'] = 'SingleToggleElement';
SINGLE_ELEMENT_TYPES['formattedtext'] = 'SingleFormattedTextElement';
SINGLE_ELEMENT_TYPES['link'] = 'SingleLinkElement';
SINGLE_ELEMENT_TYPES['datetime'] = 'SingleDateElement';
SINGLE_ELEMENT_TYPES['file'] = 'SingleFileElement';
SINGLE_ELEMENT_TYPES['video'] = 'SingleVideoElement';
SINGLE_ELEMENT_TYPES['image'] = 'SingleImageElement';
SINGLE_ELEMENT_TYPES['optionselection'] = 'SingleOptionSelectionElement';
SINGLE_ELEMENT_TYPES['reference'] = 'SingleReferenceElement';
SINGLE_ELEMENT_TYPES['category'] = 'CategoryElement';
SINGLE_ELEMENT_TYPES['location'] = 'LocationElement';
function _getElementValidator(aType) {
    var type = SINGLE_ELEMENT_TYPES[aType];
    return ('is' +
        (type && type.startsWith('Single') ? type.substr('Single'.length) : type));
}
var MULTI_ELEMENT_TYPES = {};
MULTI_ELEMENT_TYPES['text'] = 'MultiTextElement';
MULTI_ELEMENT_TYPES['number'] = 'MultiNumberElement';
MULTI_ELEMENT_TYPES['toggle'] = 'MultiToggleElement';
MULTI_ELEMENT_TYPES['formattedtext'] = 'MultiFormattedTextElement';
MULTI_ELEMENT_TYPES['link'] = 'MultiLinkElement';
MULTI_ELEMENT_TYPES['datetime'] = 'MultiDateElement';
MULTI_ELEMENT_TYPES['file'] = 'MultiFileElement';
MULTI_ELEMENT_TYPES['video'] = 'MultiVideoElement';
MULTI_ELEMENT_TYPES['image'] = 'MultiImageElement';
MULTI_ELEMENT_TYPES['optionselection'] = 'MultiOptionSelectionElement';
MULTI_ELEMENT_TYPES['reference'] = 'MultiReferenceElement';
var TYPES = {};
TYPES['text'] = 'string';
TYPES['number'] = 'number';
TYPES['toggle'] = 'boolean';
TYPES['formattedtext'] = 'string';
TYPES['link'] = 'Link';
TYPES['datetime'] = 'Date';
TYPES['file'] = 'File';
TYPES['video'] = 'Video';
TYPES['image'] = 'Image';
TYPES['optionselection'] = 'OptionSelection';
TYPES['reference'] = 'RenderingContext';
TYPES['category'] = 'Category';
TYPES['location'] = 'Location';
var AUGMENTED_TYPES = {};
AUGMENTED_TYPES['Image'] = 'SingleImageElement';
AUGMENTED_TYPES['Video'] = 'SingleVideoElement';
AUGMENTED_TYPES['File'] = 'SingleFileElement';
AUGMENTED_TYPES['Link'] = 'SingleLinkElement';
AUGMENTED_TYPES['Category'] = 'CategoryElement';
AUGMENTED_TYPES['Location'] = 'LocationElement';
var DEFAULT_VALUES = {};
DEFAULT_VALUES['text'] = "''";
DEFAULT_VALUES['number'] = '0';
DEFAULT_VALUES['toggle'] = 'false';
DEFAULT_VALUES['formattedtext'] = "''";
var BUILT_IN_TYPES = ['string', 'number', 'boolean', 'Date'];
function _createTemplate(aOptions, aLogger) {
    var rxBaseComponent = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'component.hbs'));
    var rxTypeComponent = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'type.component.hbs'));
    var rxTypeInterface = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'type.interface.hbs'));
    var rxTypeDefinition = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'type.definition.hbs'));
    var rxTypeHtml = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'type.component.html.hbs'));
    var rxTypeCss = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'type.component.css.hbs'));
    var rxLayoutComponent = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'layout.component.hbs'));
    var rxLayoutModule = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'layout.module.hbs'));
    // allow to override the template
    var layoutHtmlFile = aOptions.layoutHtmlTemplate
        ? options_1.optNormalizeDir(aOptions.layoutHtmlTemplate)
        : path_1.join(ASSETS_BASE, 'layout.component.html.hbs');
    // log this
    aLogger.next("LayoutHtmlTemplate[$layoutHtmlFile}].");
    var rxLayoutHtml = rx_hbs_1.hbsCompile(layoutHtmlFile);
    var rxLayoutCss = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'layout.component.css.hbs'));
    var rxLayoutOverview = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'layout.overview.hbs'));
    var rxLayoutExports = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'layout.export.hbs'));
    var rxModuleOverview = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'module.overview.hbs'));
    var rxModuleExports = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'module.export.hbs'));
    // return
    return {
        abstractComponent: rxBaseComponent,
        typeComponent: rxTypeComponent,
        typeInterface: rxTypeInterface,
        typeDefinition: rxTypeDefinition,
        typeHtml: rxTypeHtml,
        typeCss: rxTypeCss,
        layoutComponent: rxLayoutComponent,
        layoutModule: rxLayoutModule,
        layoutHtml: rxLayoutHtml,
        layoutCss: rxLayoutCss,
        layoutOverview: rxLayoutOverview,
        layoutExports: rxLayoutExports,
        moduleOverview: rxModuleOverview,
        moduleExports: rxModuleExports
    };
}
exports.createComponentTemplates = _createTemplate;
function _getExpression(aElement) {
    var type = aElement.allowMultipleValues
        ? _plural(aElement.elementType)
        : aElement.elementType;
    return type + '.' + aElement.key;
}
function _createTypeComponentContext(aClass, aTools) {
    // clone the type
    var cloned = lodash_1.cloneDeep(aClass.typeDef.type);
    var types = {};
    // set class name
    cloned.baseClassName = aClass.abstractComponentClass;
    cloned.className = aClass.typeComponentClass;
    cloned.baseFileName = './' + aClass.abstractComponentRef;
    cloned.templateUrl = './' + aClass.typeTemplateFile;
    cloned.styleUrl = './' + aClass.typeStyleFile;
    cloned.selector = aClass.typeSelector;
    // the imports
    cloned.imports = Object.keys(types).sort();
    return cloned;
}
function _createLayoutComponentContext(aClass, aTools, aOptions) {
    var cloned = lodash_1.cloneDeep(aClass.layout);
    // check the editable flag
    cloned.editable = aOptions.editable || false;
    // classname
    cloned.className = aClass.layoutClass;
    cloned.baseClassName = aClass.component.typeComponentClass;
    cloned.selector = aClass.layoutSelector;
    cloned.layoutTemplate = aClass.layoutTemplate;
    cloned.templateUrl = './' + aClass.layoutTemplateFile;
    cloned.styleUrl = './' + aClass.layoutStyleFile;
    // path to the layout
    var layoutPath = aClass.folder;
    var componentPath = aClass.component.folder;
    // compute the relative path
    var relPath = file_utils_2.relativePath(layoutPath, componentPath);
    cloned.baseFileName =
        './' + relPath + '/' + aClass.component.typeComponentRef;
    return cloned;
}
function _createLayoutModuleContext(aClass, aTools, aOptions) {
    // dispatch
    var ctx = _createLayoutComponentContext(aClass, aTools, aOptions);
    // augment the context
    var moduleClassName = aClass.moduleClass;
    ctx.moduleClassName = moduleClassName;
    ctx.layoutClassName = aClass.layoutClass;
    // path to the layout and module
    var layoutPath = aClass.folder;
    var modulePath = aClass.folder;
    // remember
    ctx.layoutPath = layoutPath;
    ctx.layoutModulePath = modulePath;
    // compute the relative path
    ctx.layoutFileName = './' + aClass.layoutRef;
    return ctx;
}
function _createAbstractComponentContext(aClass, aComponentReg, aTools) {
    // clone the type
    var cloned = lodash_1.cloneDeep(aClass.typeDef.type);
    // set class name
    cloned.className = aClass.abstractComponentClass;
    // set class name
    cloned.interfacesName = aClass.typeInterfaceClass;
    // target name
    cloned.interfacesFileName = aClass.typeInterfaceRef;
    // path to the type interface
    var interfacePath = aClass.folder;
    // type checks
    var typeChecks = (cloned.typeChecks = {});
    // produce the bindings
    var imports = {};
    // always add rendering context
    _registerImport(TYPE_CONTEXT, DEP_SDK, imports);
    _registerImport(TYPE_ABSTRACT_COMPONENT, DEP_SDK, imports);
    // register the type checks
    var interfaceImport = "./" + aClass.typeInterfaceRef;
    _registerImport(cloned.interfacesName, interfaceImport, imports);
    _registerImport("is" + cloned.interfacesName, interfaceImport, imports);
    _registerImport("assert" + cloned.interfacesName, interfaceImport, imports);
    // check if we have bindings
    if (cloned.elements && cloned.elements.length > 0) {
        // also add bindings
        _registerImport(TYPE_BINDING, DEP_SDK, imports);
    }
    // work on the bindings
    var rxBindings = cloned.elements.map(function (el) {
        // result
        var binding = {};
        // class name
        binding.elKey = el.key;
        binding.key = _camelCase(el.key);
        binding.observableKey = _camelCase('on ' + el.key);
        binding.expression = _getExpression(el);
        binding.comment = _toComment(el);
        binding.required = !!el.required;
        binding.optional = !!!el.required;
        // defaults
        if (!el.required) {
            if (el.allowMultipleValues) {
                binding.default = '[]';
            }
            else {
                binding.default = DEFAULT_VALUES[el.elementType];
            }
        }
        // type
        var rxTypeName;
        // check for the element type
        var bIsGroup = el.elementType === 'group';
        if (bIsGroup) {
            // type id
            var typeId = el.typeRef.id;
            // locate the type
            var rxRefType = aComponentReg
                .findTypeClass(aTools.types[typeId])
                .pipe(operators_1.shareReplay());
            // the from statement
            var rxFrom = rxRefType.pipe(operators_1.map(function (typeDef) {
                return './' +
                    file_utils_2.relativePath(interfacePath, typeDef.folder) +
                    '/' +
                    typeDef.typeElementRef;
            }));
            // the type name
            var rxType = rxRefType.pipe(operators_1.map(function (typeDef) { return typeDef.simpleTypeClass; }));
            // register the import
            var rxImportType = rxjs_1.combineLatest(rxFrom, rxType).pipe(operators_1.map(function (_a) {
                var from = _a[0], name = _a[1];
                return _registerImport(name, from, imports);
            }));
            // the type name
            rxTypeName = rxImportType.pipe(operators_1.mergeMapTo(rxType));
        }
        else {
            // type
            var type = TYPES[el.elementType];
            if (BUILT_IN_TYPES.indexOf(type) < 0) {
                _registerImport(type, DEP_SDK, imports);
            }
            // get the type
            rxTypeName = rxjs_1.of(type);
        }
        return rxTypeName.pipe(operators_1.map(function (typeName) {
            // update the type field
            binding.type = el.allowMultipleValues ? typeName + '[]' : typeName;
            // add
            return binding;
        }));
    });
    // wait
    return rx_utils_1.rxForkJoin(rxBindings).pipe(operators_1.map(function (bindings) {
        // update
        cloned.bindings = bindings;
        // organize
        var otherImports = {};
        Object.keys(imports)
            .sort()
            .forEach(function (from) { return (otherImports[from] = Object.keys(imports[from]).sort()); });
        // update
        cloned.otherImports = otherImports;
        // add the types
        // ok
        return cloned;
    }));
}
exports.createAbstractComponentContext = _createAbstractComponentContext;
function _createConstantName(aKey) {
    return lodash_1.replace(transliteration_1.kebabCase(aKey).toUpperCase(), /\-/g, '_');
}
function _registerImport(aClass, aFrom, aImports) {
    // create the hook
    var imp = aImports[aFrom] || (aImports[aFrom] = {});
    // add
    imp[aClass] = aClass;
}
function _createTypeInterfaceContext(aClass, aComponentReg, aTools) {
    // clone the type
    var typeDef = aClass.typeDef;
    var cloned = lodash_1.cloneDeep(aClass.typeDef.type);
    // set class name
    cloned.interfacesName = aClass.typeInterfaceClass;
    cloned.definitionName = aClass.typeDef.typeElementClass;
    // path to the type interface
    var interfacePath = aClass.folder;
    var definitionPath = aClass.typeDef.folder;
    // compute the relative path
    var relPath = file_utils_2.relativePath(interfacePath, definitionPath);
    var definitionRef = './' + relPath + '/' + aClass.typeDef.typeElementRef;
    // top level fields
    var expressions = {};
    // produce the bindings
    var imports = {};
    // the well known type
    _registerImport(cloned.definitionName, definitionRef, imports);
    _registerImport("is" + cloned.definitionName, definitionRef, imports);
    var rxBindings = cloned.elements.map(function (el) {
        // result
        var binding = {};
        // class name
        binding.constant = 'KEY_' + _createConstantName(el.key);
        binding.key = el.key;
        binding.observableKey = _camelCase('on ' + el.key);
        binding.expression = _getExpression(el);
        binding.comment = _toComment(el);
        binding.required = !!el.required;
        binding.optional = !!!el.required;
        // import the constant
        _registerImport(binding.constant, definitionRef, imports);
        var rxTypeName;
        var bIsGroup = el.elementType === 'group';
        if (bIsGroup) {
            // type id
            var typeId = el.typeRef.id;
            // locate the type
            var rxRefType = aComponentReg
                .findTypeClass(aTools.types[typeId])
                .pipe(operators_1.shareReplay());
            // the from statement
            var rxFrom = rxRefType.pipe(operators_1.map(function (typeDef) {
                return './' +
                    file_utils_2.relativePath(interfacePath, typeDef.folder) +
                    '/' +
                    typeDef.typeElementRef;
            }));
            // the type name
            var rxType = rxRefType.pipe(operators_1.map(function (typeDef) { return typeDef.simpleTypeClass; }));
            // register the import
            var rxImportType = rxjs_1.combineLatest(rxFrom, rxType).pipe(operators_1.map(function (_a) {
                var from = _a[0], name = _a[1];
                return _registerImport(name, from, imports);
            }));
            // the type name
            rxTypeName = rxImportType.pipe(operators_1.mergeMapTo(rxType));
        }
        else {
            var directType = TYPES[el.elementType];
            var augType = AUGMENTED_TYPES[directType];
            var expElType = augType || directType;
            if (BUILT_IN_TYPES.indexOf(expElType) < 0) {
                _registerImport(expElType, DEP_SDK, imports);
            }
            rxTypeName = rxjs_1.of(expElType);
        }
        // get the top level entry
        var expType = el.allowMultipleValues
            ? _plural(el.elementType)
            : el.elementType;
        var exp = expressions[expType] || (expressions[expType] = {});
        // handle
        return rxTypeName.pipe(operators_1.map(function (typeName) {
            // update the expression
            exp[el.key] = {
                key: el.key,
                comment: _toComment(el),
                type: el.allowMultipleValues ? typeName + '[]' : typeName,
                optional: !!!el.required
            };
            // returns the binding
            return binding;
        }));
    });
    // wait
    return rx_utils_1.rxForkJoin(rxBindings).pipe(operators_1.map(function (bindings) {
        // update
        cloned.bindings = bindings;
        // handle the imports
        var fromNg = imports[DEP_SDK] || {};
        delete fromNg[TYPE_CONTEXT];
        // the imports
        cloned.imports = Object.keys(fromNg).sort();
        cloned.expressions = expressions;
        // the other importa
        delete imports[DEP_SDK];
        // organize
        var otherImports = {};
        Object.keys(imports)
            .sort()
            .forEach(function (from) { return (otherImports[from] = Object.keys(imports[from]).sort()); });
        // update
        cloned.otherImports = otherImports;
        // ok
        return cloned;
    }));
}
exports.createTypeInterfaceContext = _createTypeInterfaceContext;
function _createTypeDefinitionContext(aTypeClass, aComponentReg, aTools) {
    // clone the type
    var typeDef = aTypeClass;
    var cloned = lodash_1.cloneDeep(typeDef.type);
    // set class name
    cloned.definitionName = typeDef.typeElementClass;
    cloned.simpleTypeName = typeDef.simpleTypeClass;
    // path to the type interface
    var definitionPath = typeDef.folder;
    // produce the bindings
    var imports = {};
    // add default import
    _registerImport('GroupElement', DEP_SDK, imports);
    _registerImport('isSingleGroupElement', DEP_SDK, imports);
    _registerImport('isMultiGroupElement', DEP_SDK, imports);
    var rxBindings = cloned.elements.map(function (el) {
        // result
        var binding = {};
        // class name
        binding.key = el.key;
        binding.constant = 'KEY_' + _createConstantName(el.key);
        binding.observableKey = _camelCase('on ' + el.key);
        binding.expression = _getExpression(el);
        binding.comment = _toComment(el);
        binding.required = !!el.required;
        binding.optional = !!!el.required;
        var rxElementName;
        var rxSimpleName;
        var bIsGroup = el.elementType === 'group';
        if (bIsGroup) {
            // type id
            var typeId = el.typeRef.id;
            // locate the type
            var rxRefType = aComponentReg
                .findTypeClass(aTools.types[typeId])
                .pipe(operators_1.shareReplay());
            // the from statement
            var rxFrom = rxRefType.pipe(operators_1.map(function (typeDef) {
                return './' +
                    file_utils_2.relativePath(definitionPath, typeDef.folder) +
                    '/' +
                    typeDef.typeElementRef;
            }));
            // the type name
            var rxNameEl = rxRefType.pipe(operators_1.map(function (typeDef) {
                return el.allowMultipleValues
                    ? typeDef.multiElementClass
                    : typeDef.singleElementClass;
            }), operators_1.shareReplay());
            // register the import
            var rxImportElement = rxjs_1.combineLatest(rxFrom, rxNameEl, rxRefType).pipe(operators_1.map(function (_a) {
                var from = _a[0], name = _a[1], typeDef = _a[2];
                // test for idempotent imports
                if (definitionPath !== typeDef.folder) {
                    _registerImport(name, from, imports);
                    _registerImport("is" + name, from, imports);
                }
            }));
            // the type name
            rxElementName = rxImportElement.pipe(operators_1.mergeMapTo(rxNameEl));
            // the type name
            var rxNameSimp = rxRefType.pipe(operators_1.map(function (typeDef) { return typeDef.simpleTypeClass; }), operators_1.shareReplay());
            // register the import
            var rxImportSimple = rxjs_1.combineLatest(rxFrom, rxNameSimp, rxRefType).pipe(operators_1.map(function (_a) {
                var from = _a[0], name = _a[1], typeDef = _a[2];
                // test for idempotent imports
                if (definitionPath !== typeDef.folder) {
                    _registerImport(name, from, imports);
                }
            }));
            // the type name
            rxSimpleName = rxImportSimple.pipe(operators_1.mergeMapTo(rxNameSimp));
        }
        else {
            // decode the type name
            var elementName = el.allowMultipleValues
                ? MULTI_ELEMENT_TYPES[el.elementType]
                : SINGLE_ELEMENT_TYPES[el.elementType];
            if (BUILT_IN_TYPES.indexOf(elementName) < 0) {
                // register the import
                _registerImport(elementName, DEP_SDK, imports);
                // import the type check
                _registerImport("is" + elementName, DEP_SDK, imports);
            }
            // decode the type name
            var simpleName = TYPES[el.elementType];
            if (BUILT_IN_TYPES.indexOf(simpleName) < 0) {
                // register the import
                _registerImport(simpleName, DEP_SDK, imports);
            }
            // ok
            rxSimpleName = rxjs_1.of(simpleName);
            // ok
            rxElementName = rxjs_1.of(elementName);
        }
        // handle
        return rx_utils_1.rxForkJoin([rxSimpleName, rxElementName]).pipe(operators_1.map(function (_a) {
            var simpName = _a[0], elName = _a[1];
            // update the binding
            binding.type = elName;
            binding.simpleTypeName = simpName;
            binding.validator = "is" + elName;
            // returns the binding
            return binding;
        }));
    });
    // wait
    return rx_utils_1.rxForkJoin(rxBindings).pipe(operators_1.map(function (bindings) {
        // update
        cloned.bindings = bindings;
        cloned.hasBindings = Object.keys(bindings).length > 0;
        // organize
        var otherImports = {};
        Object.keys(imports)
            .sort()
            .forEach(function (from) { return (otherImports[from] = Object.keys(imports[from]).sort()); });
        // update
        cloned.otherImports = otherImports;
        cloned.baseElementClass = typeDef.baseElementClass;
        cloned.singleElementClass = typeDef.singleElementClass;
        cloned.multiElementClass = typeDef.multiElementClass;
        // ok
        return cloned;
    }));
}
function _createAbstractComponent(aClass, aTools, aTemplate, aOptions, aComponentReg, aLogger) {
    // prepare the context
    var rxCtx = _createAbstractComponentContext(aClass, aComponentReg, aTools);
    // target name
    var dstFile = path_1.join(aClass.folder, aClass.abstractComponentFile);
    // log this
    aLogger.next("abstractComponent for " + aClass.typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = rxCtx.pipe(operators_1.mergeMap(function (ctx) { return aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); })); }));
    // write the content
    return newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, true); }));
}
function _createTypeInterface(aClass, aTools, aTemplate, aOptions, aComponentReg, aLogger) {
    // prepare the context
    var rxCtx = _createTypeInterfaceContext(aClass, aComponentReg, aTools);
    // target name
    var dstFile = path_1.join(aClass.folder, aClass.typeInterfaceFile);
    // log this
    aLogger.next("typeInterface for " + aClass.typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = rxCtx.pipe(operators_1.mergeMap(function (ctx) { return aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); })); }));
    // write the content
    return newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, true); }));
}
/**
 * Get the type dependencies
 *
 * @param aTypeClass
 */
function _getTypeDependencies(aTypeClass, aTools, aComponentReg) {
    // type
    var type = aTypeClass.type;
    // analyze the elements
    return rxjs_1.merge.apply(void 0, type.elements
        .filter(function (el) { return el.elementType === 'group'; })
        .map(function (el) {
        return aComponentReg.findTypeClass(aTools.types[el.typeRef.id]);
    }));
}
function _createTypeDefinition(aTypeClass, aTools, aTemplate, aOptions, aComponentReg, aLogger) {
    // check for dups
    if (!aComponentReg.registerType(aTypeClass.type)) {
        return rxjs_1.empty();
    }
    // the def
    var typeDef = aTypeClass;
    // prepare the context
    var rxCtx = _createTypeDefinitionContext(typeDef, aComponentReg, aTools);
    // target name
    var dstFile = path_1.join(typeDef.folder, typeDef.typeElementFile);
    // log this
    aLogger.next("typeDefinition for " + typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = rxCtx.pipe(operators_1.mergeMap(function (ctx) { return aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); })); }));
    // write the content
    var rxResult = newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, true); }));
    // generate all transitive types
    var rxDependencies = _getTypeDependencies(typeDef, aTools, aComponentReg).pipe(operators_1.mergeMap(function (aDepType) {
        return _createTypeDefinition(aDepType, aTools, aTemplate, aOptions, aComponentReg, aLogger);
    }));
    // merge together
    return rxjs_1.merge(rxResult, rxDependencies);
}
function _createTypeComponent(aClass, aTools, aTemplate, aOptions, aLogger) {
    // override flag
    var bOverride = !!aOptions.override;
    // prepare the context
    var ctx = _createTypeComponentContext(aClass, aTools);
    // target name
    var dstFile = path_1.join(aClass.folder, aClass.typeComponentFile);
    // log this
    aLogger.next("typeComponent for " + aClass.typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); }));
    // write the content
    return newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, bOverride); }));
}
function _createTypeHtmlContext(aClass, aTools) {
    return {
        className: aClass.typeComponentClass
    };
}
/**
 * Constructs the context to generate a layout sample
 *
 * @param aLayout   the layout
 * @param aMapping  the layout mapping
 * @param aType     the type
 * @param aTools    access to the tools
 */
function _createLayoutHtmlContext(aClass, aTools, aOptions) {
    // elements
    var el = lodash_1.cloneDeep(aClass.type.elements);
    // iterate and add the propery name
    lodash_1.forEach(el, function (e) {
        e.prop = _camelCase(e.key);
        e.onprop = _camelCase('on ' + e.key);
    });
    return {
        className: aClass.layoutClass,
        editable: !!aOptions.editable || false,
        elements: el
    };
}
function _createTypeHtml(aClass, aTools, aTemplate, aOptions, aLogger) {
    // override flag
    var bOverride = !!aOptions.override;
    // prepare the context
    var ctx = _createTypeHtmlContext(aClass, aTools);
    // target name
    var dstFile = path_1.join(aClass.folder, aClass.typeTemplateFile);
    // log this
    aLogger.next("typeTemplate for " + aClass.typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); }));
    // write the content
    return newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, bOverride); }));
}
function _createTypeCssContext(aClass, aTools) {
    return {
        className: aClass.typeComponentClass
    };
}
function _createLayoutCssContext(aClass, aTools) {
    return {
        className: aClass.layoutClass
    };
}
function _createTypeCss(aClass, aTools, aTemplate, aOptions, aLogger) {
    // override flag
    var bOverride = !!aOptions.override;
    // prepare the context
    var ctx = _createTypeCssContext(aClass, aTools);
    // target name
    var dstFile = path_1.join(aClass.folder, aClass.typeStyleFile);
    // log this
    aLogger.next("typeStyles for " + aClass.typeDef.type.name + " => " + dstFile);
    // execute the template
    var newContent = aTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); }));
    // write the content
    return newContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstFile, data, bOverride); }));
}
function _createComponent(aClass, aTools, aTemplate, aOptions, aComponentReg, aLogger) {
    // log this
    aLogger.next("creating components for [" + aClass.typeDef.type.name + "] ...");
    // construct the base class
    var baseClass = _createAbstractComponent(aClass, aTools, aTemplate.abstractComponent, aOptions, aComponentReg, aLogger);
    var typeClass = _createTypeComponent(aClass, aTools, aTemplate.typeComponent, aOptions, aLogger);
    var typeInterface = _createTypeInterface(aClass, aTools, aTemplate.typeInterface, aOptions, aComponentReg, aLogger);
    var typeDefinition = _createTypeDefinition(aClass.typeDef, aTools, aTemplate.typeDefinition, aOptions, aComponentReg, aLogger);
    var typeHtml = _createTypeHtml(aClass, aTools, aTemplate.typeHtml, aOptions, aLogger);
    var typeCss = _createTypeCss(aClass, aTools, aTemplate.typeCss, aOptions, aLogger);
    // merge the results
    return rxjs_1.merge(baseClass, typeClass, typeInterface, typeHtml, typeCss, typeDefinition);
}
/**
 * Construct the base components
 *
 * @param aTools
 * @param aSource
 * @param aTemplate
 * @param aMatcher
 * @param aOptions
 * @param aLogger
 */
function _createComponents(aTools, aTemplate, aMatcher, aOptions, aCmpReg, aLogger) {
    // log this
    aLogger.next("createComponents [" + Object.keys(aTools.types) + "] ...");
    // iterate
    return rxjs_1.of.apply(void 0, Object.keys(aTools.types)).pipe(operators_1.map(function (id) { return aTools.types[id]; }), operators_1.filter(aMatcher), operators_1.mergeMap(function (type) { return aCmpReg.findComponentClass(type); }), operators_1.mergeMap(function (clz) {
        return _createComponent(clz, aTools, aTemplate, aOptions, aCmpReg, aLogger);
    }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next('createComponents done.'); }));
}
function _createLayoutComponent(aClass, aTools, aTemplate, aOptions, aLayoutReg, aLogger) {
    // cycle check
    if (!aLayoutReg.registerLayout(aClass.layout)) {
        // do not create a layout, twice
        return rxjs_1.empty();
    }
    // override flag
    var bOverride = !!aOptions.override;
    // code
    var dstLayoutFile = path_1.join(aClass.folder, aClass.layoutFile);
    var layoutCtx = _createLayoutComponentContext(aClass, aTools, aOptions);
    var layoutModuleCtx = _createLayoutModuleContext(aClass, aTools, aOptions);
    var layoutContent = aTemplate.layoutComponent.pipe(operators_1.map(function (tmp) { return tmp(layoutCtx); }));
    var layoutModule = aTemplate.layoutModule.pipe(operators_1.map(function (tmp) { return tmp(layoutModuleCtx); }));
    var layoutFile = layoutContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstLayoutFile, data, bOverride); }));
    // log this
    aLogger.next("layoutComponent for " + aClass.type.name + " => " + dstLayoutFile);
    // module
    var dstModuleFileName = path_1.join(aClass.folder, aClass.moduleFile);
    var layoutModuleFile = layoutModule.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstModuleFileName, data, bOverride); }));
    // html
    var dstHtmlFile = path_1.join(aClass.folder, aClass.layoutTemplateFile);
    var htmlCtx = _createLayoutHtmlContext(aClass, aTools, aOptions);
    var htmlContent = aTemplate.layoutHtml.pipe(operators_1.mergeMap(function (tmp) { return rx_tidy_1.rxTidy(tmp(htmlCtx).replace(/^\s*[\r\n]/gm, '')); }));
    var htmlFile = htmlContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstHtmlFile, data, bOverride); }));
    // log this
    aLogger.next("layoutTemplate for " + aClass.type.name + " => " + dstHtmlFile);
    // css
    var dstCssFile = path_1.join(aClass.folder, aClass.layoutStyleFile);
    var cssCtx = _createLayoutCssContext(aClass, aTools);
    var cssContent = aTemplate.layoutCss.pipe(operators_1.map(function (tmp) { return tmp(cssCtx); }));
    var cssFile = cssContent.pipe(operators_1.mergeMap(function (data) { return rx_file_1.writeTextFileSafe(dstCssFile, data, bOverride); }));
    // log this
    aLogger.next("layoutStyles for " + aClass.type.name + " => " + dstCssFile);
    /*
  
      // prepare the context
      const ctx = _createTypeComponentContext(aType, aTools);
      // target name
      const fileName = _camelCase('type ' + aType.name) + '.ts';
      // execute the template
      const newContent = aTemplate.map(tmp => tmp(ctx));
      // write the content
      return combineLatest(src, newContent)
          .mergeMap(([path, data]) => writeFile(join(path, fileName), data));
      */
    // handle the module file option
    var files = [layoutFile, htmlFile, cssFile];
    if (aOptions.modules) {
        files.push(layoutModuleFile);
        // log this
        aLogger.next("layoutModule for " + aClass.type.name + " => " + dstModuleFileName);
    }
    // merge
    return rxjs_1.merge.apply(void 0, files);
}
/**
 * Get all layouts for a particular layout mapping
 *
 * @param aMapping  the mapping
 * @param aTools    the tools
 *
 * @return the observable with the layouts
 */
function _getLayouts(aMapping, aTools) {
    var mappings = aMapping.mappings;
    return rxjs_1.Observable.create(function (observer) {
        mappings.forEach(function (mapping) {
            var layouts = mapping.layouts;
            layouts.forEach(function (layout) { return observer.next(aTools.layouts[layout.id]); });
        });
        // done
        observer.complete();
    });
}
function _getAllLayouts(aMapping, aTools) {
    // iterate
    return rxjs_1.of.apply(void 0, Object.keys(aTools.layoutMappings)).pipe(operators_1.map(function (id) { return aTools.layoutMappings[id]; }), operators_1.mergeMap(function (mapping) { return _getLayouts(mapping, aTools); }), operators_1.filter(function (layout) { return layout != null && layout.templateType === 'angular'; }), operators_1.distinct(function (layout) { return layout.id; }), 
    // is name unique??
    operators_1.distinct(function (layout) { return layout.name; }));
}
function _createLayout(aMapping, aTools, aTemplate, aOptions, aLayoutReg, aLogger) {
    // decode the type
    var typeId = aMapping.type.id;
    var type = aTools.types[typeId];
    // check
    if (!type) {
        return rxjs_1.empty();
    }
    // check the layouts
    return _getLayouts(aMapping, aTools).pipe(operators_1.filter(function (layout) { return layout != null && layout.templateType === 'angular'; }), operators_1.distinct(function (layout) { return layout.id; }), 
    // is name unique??
    operators_1.distinct(function (layout) { return layout.name; }), 
    // resolve the layout class
    operators_1.mergeMap(function (layout) { return aLayoutReg.findLayoutClass(layout, aMapping, type); }), 
    // produce the layout
    operators_1.mergeMap(function (clz) {
        return _createLayoutComponent(clz, aTools, aTemplate, aOptions, aLayoutReg, aLogger);
    }));
}
var regex = /(?:.|\n|\r)*@LayoutComponent(?:.|\n|\r)*@Component(?:.|\n|\r)*class\s+(\w+)\s+(?:.|\n|\r)*/;
var layoutComponent = /@LayoutComponent/;
var component = /@Component/;
var className = /class\s+(\w+)\s+/;
function _stripComment(aLine) {
    // replace all after "//"
    var idx = aLine.indexOf('//');
    var line = idx >= 0 ? aLine.substr(0, idx) : aLine;
    // replace /* */ like comments
    return line.replace(/(\/\*.*?\*\/)/g, '');
}
function _stripSuffix(aValue) {
    var idx = aValue.lastIndexOf('.');
    if (idx >= 0) {
        return aValue.substr(0, idx);
    }
    return aValue;
}
var count = 0;
function _moduleDescriptorFromClassDescriptor(aClassDescriptor) {
    // get the name of the module
    var absPath = aClassDescriptor.abs;
    if (absPath.endsWith('Layout.ts')) {
        // extract
        var parsedPath = path_1.parse(absPath);
        var layoutBase = parsedPath.name;
        var moduleBase = layoutBase.substr(0, layoutBase.length - 'Layout'.length) + 'Module';
        var moduleName = moduleBase + '.ts';
        var layoutPath = aClassDescriptor.path;
        var modulePath = layoutPath.substr(0, layoutPath.length - 'Layout'.length) + 'Module';
        // test if that exists
        var absModulePath = path_1.join(parsedPath.dir, moduleName);
        // test if it exist
        return rx_walk_1.rxStats(absModulePath).pipe(operators_1.catchError(function (error) { return rxjs_1.empty(); }), operators_1.mapTo({
            root: parsedPath.dir,
            path: modulePath,
            class: lodash_1.upperFirst(moduleBase),
            abs: absModulePath
        }));
    }
    // nothing
    return rxjs_1.empty();
}
function _extractClass(aFile) {
    // match by line
    var rs = graceful_fs_1.createReadStream(aFile.absPath, 'utf-8');
    // parse the lines
    return rx_lines_1.rxReadLines(rs).pipe(operators_1.map(_stripComment), operators_1.skipWhile(function (line) { return !layoutComponent.test(line); }), operators_1.skipWhile(function (line) { return !component.test(line); }), operators_1.filter(function (line) { return className.test(line); }), operators_1.map(function (line) { return ({
        root: aFile.root,
        path: './' + _stripSuffix(lodash_1.replace(aFile.relPath, /\\/g, '/')),
        class: className.exec(line)[1],
        abs: aFile.absPath
    }); }));
}
function _isRelative(aPath) {
    return !!aPath && (aPath.startsWith('./') || aPath.startsWith('../'));
}
function _distinctClassDescriptor(aDesc) {
    return aDesc.abs;
}
function _doExtractBaseClass(aConfig, aCycle) {
    // test if we know this
    if (aCycle[aConfig.abs]) {
        return rxjs_1.empty();
    }
    aCycle[aConfig.abs] = aConfig;
    // match by line
    var rs = graceful_fs_1.createReadStream(aConfig.abs, 'utf-8');
    var parsedPath = path_1.parse(aConfig.abs);
    // parse the lines
    var rxDoc = rx_lines_1.rxReadLines(rs).pipe(operators_1.toArray(), operators_1.map(function (lines) { return lines.join('\n'); }), operators_1.shareReplay());
    // find the base classes
    var rgBaseClasses = new RegExp("class\\s+" + escapeStringRegexp(aConfig.class) + "\\s+extends\\s+([^{]+)", 'g');
    var rxBaseClasses = rxDoc.pipe(operators_1.map(function (doc) { return rgBaseClasses.exec(doc); }), operators_1.filter(function (res) { return !!res && res.length > 1; }), operators_1.map(function (res) { return res[1]; }), operators_1.mergeMap(function (res) { return rxjs_1.from(res.split(',').map(function (cls) { return cls.trim(); })); }), operators_1.mergeMap(function (cls) {
        // locate the import location
        var rgImport = new RegExp("import\\s+{[\\s,\\w]*" + escapeStringRegexp(cls) + "[\\s,\\w]*}\\s+from\\s+['\"]([^'\"]+)['\"]", 'g');
        return rxDoc.pipe(operators_1.mergeMap(function (doc) {
            var imp = rgImport.exec(doc);
            if (!!imp && imp.length > 1) {
                // extract the path
                var relPath = imp[1].trim();
                if (_isRelative(relPath)) {
                    // resolve against the current path
                    var absImport = path_1.normalize(path_1.join(parsedPath.dir, relPath + '.ts'));
                    var relImport = './' +
                        _stripSuffix(lodash_1.replace(path_1.relative(aConfig.root, absImport), /\\/g, '/'));
                    // recurse
                    return _doExtractBaseClass({
                        root: aConfig.root,
                        path: relImport,
                        class: cls,
                        abs: absImport
                    }, aCycle).pipe(operators_1.distinct(_distinctClassDescriptor));
                }
            }
            // nothing
            return rxjs_1.empty();
        }));
    }));
    // done
    return rxBaseClasses.pipe(operators_1.startWith(aConfig));
}
function _createOverview(aSource, aTemplate, aOptions, aLogger) {
    // done
    return rxjs_1.defer(function () {
        // log this
        aLogger.next("_createOverview ...");
        // read all typescript files
        var tsFiles = aSource.pipe(operators_1.tap(function (src) { return aLogger.next("_createOverview [src=" + src + "]."); }), operators_1.mergeMap(rx_walk_1.walkFiles), operators_1.filter(function (f) { return f.stats.isFile(); }), operators_1.distinct(function (f) { return f.absPath; }), operators_1.filter(function (f) { return f.absPath.endsWith('.ts'); }));
        // read the class names of these files
        var rxClasses = tsFiles.pipe(operators_1.mergeMap(_extractClass), operators_1.tap(function (c) { return aLogger.next("Class [" + c.class + "], [" + c.path + "]."); }), operators_1.share());
        // read the layouts
        var rxSortedClasses = rxClasses.pipe(operators_1.toArray(), 
        // order the imports
        operators_1.map(function (a) { return a.sort(function (l, r) { return l.class.localeCompare(r.class); }); }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next("files done."); }), operators_1.shareReplay());
        var rxModuleClasses = rxClasses.pipe(operators_1.mergeMap(_moduleDescriptorFromClassDescriptor), operators_1.tap(function (c) { return aLogger.next("Module [" + c.class + "]."); }), operators_1.shareReplay());
        var rxSortedModuleClasses = rxModuleClasses.pipe(operators_1.toArray(), 
        // order the imports
        operators_1.map(function (a) { return a.sort(function (l, r) { return l.class.localeCompare(r.class); }); }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next("files done."); }), operators_1.shareReplay());
        var baseCycles = {};
        // imports
        var rxSortedImports = rxClasses.pipe(operators_1.mergeMap(function (cls) { return _doExtractBaseClass(cls, baseCycles); }), operators_1.distinct(_distinctClassDescriptor), operators_1.toArray(), 
        // order the imports
        operators_1.map(function (a) { return a.sort(function (l, r) { return l.class.localeCompare(r.class); }); }), operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next("base classes done."); }), operators_1.shareReplay());
        // template
        var overviewTemplate = aTemplate.layoutOverview;
        var overviewRes = rxjs_1.combineLatest(overviewTemplate, rxSortedClasses).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        var exportTemplate = aTemplate.layoutExports;
        var exportRes = rxjs_1.combineLatest(exportTemplate, rxSortedImports).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        var moduleOverviewTemplate = aTemplate.moduleOverview;
        var moduleOverviewRes = rxjs_1.combineLatest(moduleOverviewTemplate, rxSortedModuleClasses).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        var moduleExportTemplate = aTemplate.moduleExports;
        var moduleExportRes = rxjs_1.combineLatest(moduleExportTemplate, rxSortedModuleClasses).pipe(operators_1.map(function (_a) {
            var tmp = _a[0], ctx = _a[1];
            return tmp(ctx);
        }));
        // save overview
        var overviewFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, constants_1.LAYOUTS_FILE); }));
        var overviewWritten = rxjs_1.combineLatest(overviewFile, overviewRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        // save exports
        var exportsFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, constants_1.LAYOUTS_EXPORT_FILE); }));
        var exportsWritten = rxjs_1.combineLatest(exportsFile, exportRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        // save overview
        var modulesOverviewFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, constants_1.MODULES_FILE); }));
        var modulesOverviewWritten = rxjs_1.combineLatest(modulesOverviewFile, moduleOverviewRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        // save exports
        var moduleExportsFile = aSource.pipe(operators_1.map(function (src) { return path_1.join(src, constants_1.MODULES_EXPORT_FILE); }));
        var moduleExportsWritten = rxjs_1.combineLatest(moduleExportsFile, moduleExportRes).pipe(operators_1.mergeMap(function (_a) {
            var dst = _a[0], data = _a[1];
            return rx_file_1.writeTextFileSafe(dst, data, true);
        }));
        // total files
        var total = [overviewWritten, exportsWritten];
        aLogger.next("Wrting modules [" + aOptions.modules + "] ...");
        // add modules
        if (aOptions.modules) {
            total.push(modulesOverviewWritten, moduleExportsWritten);
        }
        return rxjs_1.merge.apply(void 0, total).pipe(operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next('createOverview done.'); }));
    });
}
exports.createComponentOverview = _createOverview;
function _isNotNil(aMapping) {
    return !!aMapping;
}
function _getTypeFromMapping(aMapping, aTools) {
    // get the ID
    if (aMapping.type && aMapping.type.id) {
        // get the type
        return aTools.types[aMapping.type.id];
    }
}
function _createLayouts(aTools, aSource, aTemplate, aMatcher, aOptions, aLayoutReg, aLogger) {
    // log this
    aLogger.next("createLayouts [" + Object.keys(aTools.layoutMappings) + "]");
    // iterate
    var layouts = rxjs_1.of.apply(void 0, Object.keys(aTools.layoutMappings)).pipe(operators_1.map(function (id) { return aTools.layoutMappings[id]; }), operators_1.filter(_isNotNil), operators_1.filter(function (mapping) { return aMatcher(_getTypeFromMapping(mapping, aTools)); }), operators_1.mergeMap(function (mapping) {
        return _createLayout(mapping, aTools, aTemplate, aOptions, aLayoutReg, aLogger);
    }));
    var overview = _createOverview(aSource, aTemplate, aOptions, aLogger);
    // concat
    return rxjs_1.concat(layouts, overview).pipe(operators_1.tap(lodash_1.noop, undefined, function () { return aLogger.next('createLayouts done.'); }));
}
function _rewriteAppModule(aLayoutExports, aModuleFile, aOptions, aLogger) {
    // just dispatch
    return !aOptions.skipRegistration
        ? ts_add_layout_1.addLayoutToModule(aLayoutExports, aModuleFile).pipe(operators_1.catchError(function () { return rxjs_1.empty(); }))
        : rxjs_1.empty();
}
exports.rewriteAppModule = _rewriteAppModule;
function _matches(aName, aMatcher, aLogger) {
    // check
    var result = aMatcher(aName);
    aLogger.next("Matching [" + aName + "] results in [" + result + "].");
    return result;
}
/**
 * Selects components by applying a matching function
 *
 * @param aMatcher  the matcher
 * @param aLogger   the logger
 */
function _byMatcher(aMatcher, aLogger) {
    // execute the check
    return function (aType) {
        return aType && aType.name && _matches(aType.name, aMatcher, aLogger);
    };
}
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create components --data DATA_FOLDER --src SOURCE_CODE_FOLDER
 */
var CreateComponentsCommand = /** @class */ (function (_super) {
    __extends(CreateComponentsCommand, _super);
    function CreateComponentsCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateComponentsCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreateComponentsCommand.COMMAND, CreateComponentsCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_MODULE + " <MODULE>", "[--" + constants_1.OPTION_MODULES + "]", "[--" + constants_1.OPTION_FLAT + "]", "[--" + constants_1.OPTION_SCSS + "]", "[--" + constants_1.OPTION_EDITABLE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_COMPONENT_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_MODULE + " <MODULE>", ':', help_1.HELP_MODULE_FILE);
        console.log("--" + constants_1.OPTION_MODULES, ':', help_1.HELP_MODULES);
        console.log("--" + constants_1.OPTION_FLAT, ':', help_1.HELP_FLAT);
        console.log("--" + constants_1.OPTION_SCSS, ':', help_1.HELP_SCSS);
        console.log("--" + constants_1.OPTION_EDITABLE, ':', help_1.HELP_EDITABLE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreateComponentsCommand.prototype.process = function () {
        var _this = this;
        return rxjs_1.defer(function () {
            // current options
            var opts = _this.options;
            // the directories
            var dataDirOpt = _this.getDataDir();
            var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || path_1.join(_this.getAppDir(), 'app'));
            // the mkdirp calls
            var mkdirp = rx_copy_1.rxMkDirs();
            // fallback source directories
            var srcDirs = (file_utils_1.arrayOf(opts.src) || [srcDirOpt]).map(options_1.optNormalizeDir);
            var rxSrcDir = mkdirp(srcDirOpt);
            var rxSrcDirs = rxjs_1.concat.apply(void 0, srcDirs.map(function (dir) { return mkdirp(dir); })).pipe(operators_1.toArray(), operators_1.shareReplay(1));
            // module file
            var moduleFile = opts.module || path_1.join(srcDirOpt, constants_1.APP_MODULE_FILE);
            var layoutsFile = path_1.join(srcDirOpt, constants_1.LAYOUTS_FILE);
            // logger
            var logger = _this.getLogger();
            // the compiles template
            var templates = _createTemplate(opts, logger);
            // matcher
            var matcher = wchtools_1.wchToolsGetMatcher(_this.options);
            var typeMatcher = _byMatcher(matcher, logger);
            // the tools
            var rxWchTools = _this.getWchTools();
            // log the name
            logger.next("Data dir [" + dataDirOpt + "], dst dir [" + srcDirOpt + "], src dirs [" + srcDirs + "], ngModule [" + moduleFile + "], layouts [" + layoutsFile + "].");
            // the registries
            var typeReg = new type_reg_1.TypeRegistry(rxSrcDirs, opts, mkdirp, logger);
            var cmpReg = new component_reg_1.ComponentRegistry(rxSrcDirs, opts, mkdirp, typeReg, logger);
            var layoutReg = new layout_reg_1.LayoutRegistry(rxSrcDirs, opts, mkdirp, cmpReg, logger);
            // attach to the tools
            return rxWchTools.pipe(operators_1.mergeMap(function (aWchTools) {
                return rxjs_1.concat(_createComponents(aWchTools, templates, typeMatcher, opts, cmpReg, logger), _createLayouts(aWchTools, rxSrcDir, templates, typeMatcher, opts, layoutReg, logger), _rewriteAppModule(layoutsFile, moduleFile, opts, logger));
            }), operators_1.distinct());
        });
    };
    CreateComponentsCommand.TYPE = 'components';
    CreateComponentsCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateComponentsCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateComponentsCommand = CreateComponentsCommand;
