"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var mime_1 = require("mime");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var json_1 = require("./../utils/json");
var revision_utils_1 = require("./../utils/revision.utils");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var create_asset_command_1 = require("./create.asset.command");
/**
 * Iterates over all image files
 *
 * @param aSrcFolder    the source folder
 * @return all image files
 */
function _getImages(aSrcFolder) {
    // walk
    return rx_walk_1.walkFiles(aSrcFolder).pipe(operators_1.filter(function (f) { return f.stats.isFile(); }), operators_1.filter(function (f) { return f && (mime_1.getType(f.absPath) || 'application/octet-stream').startsWith('image/'); }), operators_1.map(function (f) { return f.absPath; }));
}
function _loadTemplate() {
    return rx_walk_1.readFile(path_1.join(__dirname, '..', '..', '..', 'assets', 'image.content.json')).pipe(operators_1.map(function (data) { return JSON.parse(data); }));
}
function _createImage(aTypeId, aSrcImage, aContentTemplate, aDataDir, aLogger) {
    // construct the asset
    var parsedName = path_1.parse(aSrcImage);
    var assetName = parsedName.base;
    // template
    var contentId = uuid_1.v4();
    var tmp = lodash_1.cloneDeep(aContentTemplate);
    tmp.id = contentId;
    tmp.name = assetName;
    tmp.typeId = aTypeId;
    // target filename
    var tgtName = path_1.join(aDataDir, 'content', contentId + "_cmd.json");
    var asset = create_asset_command_1.writeAsset(aSrcImage, assetName, aDataDir, aLogger).pipe(operators_1.shareReplay());
    // load the asset descriptor
    var assetDesc = asset.pipe(operators_1.filter(function (name) { return name.endsWith('_amd.json'); }), operators_1.mergeMap(function (f) { return rx_walk_1.readFile(f); }), operators_1.map(function (data) { return JSON.parse(data); }), operators_1.shareReplay());
    // get the ID
    var assetId = assetDesc.pipe(operators_1.map(function (j) { return j.id; }));
    // construct the type
    var content = assetDesc.pipe(operators_1.map(function (desc) {
        // update with some metadata
        tmp.elements.image.renditions = {
            'default': {
                'renditionId': desc.renditions.default.id,
                'source': desc.renditions.default.source
            }
        };
        tmp.elements.image.asset.resourceUri = desc.renditions.default.source;
        tmp.elements.image.asset.fileName = desc.fileName;
        tmp.elements.image.asset.fileSize = desc.fileSize;
        tmp.elements.image.asset.mediaType = desc.mediaType;
    }), operators_1.mergeMap(function () { return assetId.pipe(function (aid) {
        // update the type
        tmp.elements.image.asset.id = aid;
        tmp.thumbnail.id = aid;
        // ok
        return json_1.canonicalizeJSON(revision_utils_1.addRevisionId(tmp));
    }); }), operators_1.mergeMap(function (t) { return rx_walk_1.writeFile(tgtName, JSON.stringify(t, undefined, 2)); }));
    // merge
    var all = rxjs_1.merge(asset, content).pipe(operators_1.shareReplay());
    // log when complete
    all.subscribe(undefined, undefined, function () { return aLogger.next("Image [" + aSrcImage + "]."); });
    // ok
    return all;
}
function _findTypeId(aTypeName, aTools) {
    // locate the type
    var types = aTools.types;
    return Object.keys(types).find(function (id) { return (id === aTypeName) || (types[id].name === aTypeName); }) || aTypeName;
}
function _writeImages(aTypeName, aSrcDir, aDataDir, aWchTools, aLogger) {
    // log this
    aLogger.next("Writing images ...");
    // locate the type id
    var typeId = _findTypeId(aTypeName, aWchTools);
    // load the template
    var tmp = _loadTemplate();
    // load the image
    return tmp.pipe(operators_1.mergeMap(function (tmpl) { return _getImages(aSrcDir).pipe(operators_1.mergeMap(function (img) { return _createImage(typeId, img, tmpl, aDataDir, aLogger); })); }));
}
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create images --data DATA_FOLDER --type type --src SOURCE_FOLDER
 */
var CreateImagesCommand = /** @class */ (function (_super) {
    __extends(CreateImagesCommand, _super);
    function CreateImagesCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateImagesCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        // current options
        var opts = this.options;
        var dataDirOpt = this.getDataDir();
        var rxDataDir = this.rxMkdirp(dataDirOpt);
        // source
        var sourceDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || '.');
        // type
        var type = opts.type || 'Photo';
        // load the tools
        var rxWchTools = this.getWchTools();
        // log this
        logger.next("Data dir [" + dataDirOpt + "], source files [" + sourceDirOpt + "], content type [" + type + "].");
        // load wchtools
        return rxjs_1.combineLatest(rxDataDir, rxWchTools).pipe(operators_1.mergeMap(function (_a) {
            var aDataDir = _a[0], aWchTools = _a[1];
            return _writeImages(type, sourceDirOpt, aDataDir, aWchTools, logger);
        }));
    };
    CreateImagesCommand.TYPE = 'images';
    CreateImagesCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateImagesCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateImagesCommand = CreateImagesCommand;
