import { Observable, Subject } from 'rxjs';
import { Options } from './../options';
export interface Credentials {
    username: string;
    password: string;
}
export interface WchToolsOptions extends Credentials {
    baseUrl: string;
}
export interface IdMapping<T> {
    [key: string]: T;
}
export interface SiteNode {
    children?: SiteNode[];
    data?: any;
}
export interface WchTools {
    root: string;
    idToFile: IdMapping<string>;
    layoutMappings: IdMapping<any>;
    layouts: IdMapping<any>;
    types: IdMapping<any>;
    imageProfiles: IdMapping<any>;
    content: IdMapping<any>;
    sites: IdMapping<SiteNode>;
    rxAssets: Observable<IdMapping<any>>;
    rxRenditions: Observable<IdMapping<any>>;
}
declare function _read(aDir: string): Observable<WchTools>;
/**
 * Makes sure our path ends with a proper trailing slash
 */
declare function _ensureTrailingSlash(aUrl: string): string;
export declare const KEY_API_URL = "x-ibm-dx-tenant-base-url";
export declare const KEY_USERNAME = "username";
export declare const FILE_WCHTOOLS = ".wchtoolsoptions";
export declare const FILE_WCHTOOLS_JSON = ".wchtoolsoptions.json";
declare function _getApiBaseUrl(aOptions: Options, aDataDir?: string): Observable<string>;
export declare type Matcher = (key: string) => boolean;
declare function _getPattern(aOptions: Options): Matcher;
declare function _storeCredentials(aWchToolsOptions: WchToolsOptions): Observable<string>;
declare function _getCredentials(aApiUrl: string): Observable<Credentials>;
declare function _getWchToolsOptions(aOptions: Options, aDataDir?: string): Observable<WchToolsOptions>;
declare function _hidePassword(aArgs: string[]): string[];
declare function _executeWchTools(aDataDir: string, aArgs: string[], aOptions: Options, aConfig: WchToolsOptions, aLogger: Subject<string>): Observable<string>;
declare function _getDeliveryUrlFromApiUrl(aApiUrl: string): string;
declare function _createUrl(aContextRoot: string, aOpts: WchToolsOptions): Observable<string>;
export { _read as readWchTools, _getApiBaseUrl as wchToolsGetApiBaseUrl, _getPattern as wchToolsGetMatcher, _getCredentials as wchToolsGetCredentials, _getWchToolsOptions as wchToolsGetOptions, _executeWchTools as wchExecuteWchTools, _getDeliveryUrlFromApiUrl as wchDeliveryUrlFromApiUrl, _storeCredentials as wchStoreCredentials, _ensureTrailingSlash as ensureTrailingSlash, _createUrl as wchGetIndexUrl, _hidePassword as wchHidePassword };
