/* Copyright IBM Corp. 2018 */
import { join, normalize, resolve } from 'path';
import { Observable } from 'rxjs';

import { rxCompressDir } from '../utils/rx.compress';
import { CMD_COMPRESS, OPTION_DATA, OPTION_NAME, OPTION_PKG_DIR, OPTION_ROOT, OPTION_VERBOSE } from './../constants';
import { Options } from './../options';
import { configTrimSlashes } from './../utils/config';
import { AbstractCommand } from './abstract.command';
import { Command } from './command';
import { HELP_APPLICATION_NAME, HELP_DATA_DIRECTORY, HELP_PKG_DIR, HELP_VERBOSE } from './help';


export class CompressAssetsCommand extends AbstractCommand implements Command {

    static readonly TYPE = 'assets';
    static readonly COMMAND = CMD_COMPRESS;

    constructor(options: Options) {
        super(options);
    }

    printHelp(): void {
        // log the command line
        console.log(CompressAssetsCommand.COMMAND, CompressAssetsCommand.TYPE,
            `--${OPTION_DATA} <DATA>`,
            `[--${OPTION_NAME} <NAME>]`,
            `[--${OPTION_PKG_DIR} <PKG>]`,
            `[--${OPTION_ROOT} <ROOT>]`,
            `[--${OPTION_VERBOSE}]`);
        // log the parameters
        console.log(`--${OPTION_DATA} <DATA>`, ':', HELP_DATA_DIRECTORY);
        console.log(`--${OPTION_DATA} <DATA>`, ':', HELP_DATA_DIRECTORY);
        console.log(`--${OPTION_PKG_DIR} <PKG>`, ':', HELP_PKG_DIR);
        console.log(`--${OPTION_NAME} <NAME>`, ':', HELP_APPLICATION_NAME);
        console.log(`--${OPTION_VERBOSE}`, ':', HELP_VERBOSE);
    }

    process(): Observable<string> {
        // the options
        const opts = this.getOptions();
        const appName = opts.name || this.getAppName();
        const contextRoot = `/${configTrimSlashes(opts.root || this.getContextRoot(appName))}`;
        // logger
        const logger = this.getLogger();
        // the directories
        const dataDir = this.getDataDir();
        // name of the target directory for the main files
        const assetDir = resolve(normalize(join(dataDir, 'assets', appName)));

        // log this
        logger.next(`Compressing [${assetDir}] ...`);

        // compress
        const rxResult = rxCompressDir(assetDir);

        // ok
        return rxResult;
    }
}

