/* Copyright IBM Corp. 2017 */
import { Options } from './../options';
import { AddBaseCommand } from './add.base.command';
import { AddCopyrightCommand } from './add.copyright';
import { AddEsiPolyfillCommand } from './add.esi.polyfill.command';
import { AddFaviconCommand } from './add.favicon.command';
import { AddLibraryCommand } from './add.library.command';
import { AddPreloadCommand } from './add.preload.command';
import { AddWebAppManifestCommand } from './add.webappmanifest.command';
import { ApplicationPullCommand } from './application.pull.command';
import { ApplicationPurgeCommand } from './application.purge.command';
import { ApplicationPushCommand } from './application.push.command';
import { BuildDevelopmentCommand } from './build.development.command';
import { BuildProductionCommand } from './build.production.command';
import { CanonicalizeAssetsCommand } from './canonicalize.assets.command';
import { CleanupAssetsCommand } from './cleanup.assets.command';
import { Command } from './command';
import { CompressAssetsCommand } from './compress.assets.command';
import { CopyBuildCommand } from './copy.build.command';
import { CopyContentCommand } from './copy.content.command';
import { CopyDataCommand } from './copy.data.command';
import { CreateAssetCommand } from './create.asset.command';
import { CreateComponentsCommand } from './create.components.command';
import { CreateFromArchiveCommand } from './create.from.archive.command';
import { CreateImagesCommand } from './create.images.command';
import { CreateLayoutCommand } from './create.layout.command';
import { CreateLayoutsCommand } from './create.layouts.command';
import { CreateMappingsCommand } from './create.mappings.command';
import { CreatePageTypeCommand } from './create.page.type.command';
import { CreateProjectCommand } from './create.project.command';
import { CreateSiteCommand } from './create.site.command';
import { CreateWebAppManifestCommand } from './create.webappmanifest.command';
import { EmptyCommand } from './empty';
import { NormalizeJsonCommand } from './normalize.json.command';
import { RegisterLayoutComponentsCommand } from './register.layoutcomponents.command';
import { RunWchToolsCommand } from './run.wchtools.command';
import { ShowGraphCommand } from './show.graph.command';
import { ShowTypesCommand } from './show.types.command';
import { ShowUrlCommand } from './show.url.command';
import { ShowVersionCommand } from './show.version.command';
import { StoreCredentialsCommand } from './store.credentials.command';
import { CreateReactCommand } from './create.react.command';
import { CreateVueCommand } from './create.vue.command';

interface CommandConstructor {
    new(opts: Options): Command;
}

interface CommandByType {
    [type: string]: CommandConstructor;
}

interface Commands {
    [cmd: string]: CommandByType;
}

function _register(aCommand: string, aType: string, aConstructor: CommandConstructor, aMap: Commands): void {
    // types
    const byType = aMap[aCommand] || (aMap[aCommand] = {});
    byType[aType] = aConstructor;
}

export class CommandFactory {

    private commands: Commands = {};

    constructor() {
        const cmd = this.commands;
        _register(CreateReactCommand.COMMAND, CreateReactCommand.TYPE, CreateReactCommand, cmd);
        _register(CreateVueCommand.COMMAND, CreateVueCommand.TYPE, CreateVueCommand, cmd);
        _register(CreateLayoutCommand.COMMAND, CreateLayoutCommand.TYPE, CreateLayoutCommand, cmd);
    }

    create(options: Options): Command {
        // get the type
        const cmd = options.command || 'help';
        const type = options.commandType || 'help';
        // lookup
        const byType = this.commands[cmd] || {};
        const creator: CommandConstructor = byType[type] || EmptyCommand;
        // construct the command
        return new creator(options);
    }
}