import { Observable, Subject } from 'rxjs';
import { mergeMap, tap } from 'rxjs/operators';

import { CMD_STORE, OPTION_PASSWORD, OPTION_URL, OPTION_USER } from './../constants';
import { Options } from './../options';
import { wchStoreCredentials } from './../utils/wchtools';
import { AbstractCommand } from './abstract.command';
import { Command } from './command';
import { HELP_API_URL, HELP_PASSWORD, HELP_USERNAME } from './help';

/* Copyright IBM Corp. 2017 */
function _logCredentials(aCredentials: any, aLogger: Subject<string>) {
    // log the credentials
    const logCred = Object.assign({}, aCredentials);
    delete logCred.password;
    // log
    aLogger.next(`Credentials ${JSON.stringify(logCred)}`);
}

export class StoreCredentialsCommand extends AbstractCommand implements Command {

    static readonly TYPE = 'credentials';
    static readonly COMMAND = CMD_STORE;

    constructor(private options: Options) {
        super(options);
    }

    printHelp(): void {
        // log the command line
        console.log(StoreCredentialsCommand.COMMAND, StoreCredentialsCommand.TYPE,
            `--${OPTION_URL} <URL>`,
            `--${OPTION_USER} <USER>`,
            `--${OPTION_PASSWORD} <PASSWORD>`);
        // log the parameters
        console.log(`--${OPTION_URL}`, ':', HELP_API_URL);
        console.log(`--${OPTION_USER}`, ':', HELP_USERNAME);
        console.log(`--${OPTION_PASSWORD}`, ':', HELP_PASSWORD);
    }

    process(): Observable<string> {

        // logger
        const logger = this.getLogger();

        // load the options
        return this.getWchToolsOptions().pipe(
            tap(tools => _logCredentials(tools, logger)),
            mergeMap(wchStoreCredentials)
        );
    }
}
