"use strict";
exports.__esModule = true;
var _isArray = Array.isArray;
var _keys = Object.keys;
/**
 * Tests for a plain object
 *
 * @param aObject   the object to test
 */
function _isPlainObject(aObject) {
    return aObject && !aObject.prototype && (typeof aObject === 'object');
}
var KEY_WEIGHTS = {
    'name': 1,
    'description': 2,
    'id': 3,
    'classification': 4
};
function _compareNumber(aLeft, aRight) {
    return (aLeft < aRight) ? -1 : (aLeft > aRight) ? +1 : 0;
}
function _getKey(aName) {
    return KEY_WEIGHTS[aName] || Number.MAX_SAFE_INTEGER;
}
function _compareName(aLeft, aRight) {
    // first by key
    var c = _compareNumber(_getKey(aLeft), _getKey(aRight));
    if (c === 0) {
        c = aLeft.localeCompare(aRight);
    }
    // ok
    return c;
}
function _canonicalize(aData) {
    // handle
    if (_isArray(aData)) {
        var copy_1 = [];
        aData.forEach(function (v) { return copy_1.push(_canonicalize(v)); });
        return copy_1;
    }
    if (_isPlainObject(aData)) {
        // sort the keys
        var copy_2 = {};
        _keys(aData).sort(_compareName).forEach(function (k) { return copy_2[k] = _canonicalize(aData[k]); });
        return copy_2;
    }
    // nothing to do
    return aData;
}
exports.canonicalizeJSON = _canonicalize;
