"use strict";
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var fs_extra_1 = require("fs-extra");
var graceful_fs_1 = require("graceful-fs");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var tmp_1 = require("tmp");
var verror_1 = require("verror");
function rxGetFileDescriptor(aFile, aRoot) {
    return _rxStats(aFile).pipe(operators_1.map(function (stat) { return ({
        root: aRoot,
        stats: stat,
        absPath: aFile,
        relPath: aFile.substring(aRoot.length + 1)
    }); }));
}
exports.rxGetFileDescriptor = rxGetFileDescriptor;
function _doWalkFiles(aRoot, aBaseDir) {
    return rxjs_1.Observable.create(function (observer) {
        // check the type
        var absPath = path_1.join(aRoot, aBaseDir);
        graceful_fs_1.stat(absPath, function (statErr, stats) {
            // handle the error
            if (statErr) {
                if (statErr.code === 'ENOENT') {
                    observer.complete();
                }
                else {
                    observer.error(statErr);
                }
            }
            else {
                // output the current entry
                observer.next({ root: aRoot, absPath: absPath, relPath: aBaseDir, stats: stats });
                if (stats.isDirectory()) {
                    // read the children
                    graceful_fs_1.readdir(path_1.join(aRoot, aBaseDir), function (dirErr, files) {
                        // handle
                        if (dirErr) {
                            observer.error(dirErr);
                        }
                        else {
                            // list this
                            rxjs_1.merge.apply(void 0, files.map(function (f) { return _doWalkFiles(aRoot, path_1.join(aBaseDir, f)); })).subscribe(observer);
                        }
                    });
                }
                else {
                    // done
                    observer.complete();
                }
            }
        });
    });
}
function _walkFiles(aRoot) {
    return _doWalkFiles(aRoot, '');
}
exports.walkFiles = _walkFiles;
exports.rxWalkFiles = _walkFiles;
function _writeFileIfChanged(aPath, aData) {
    return rxjs_1.Observable.create(function (observer) {
        // test if the file exists
        graceful_fs_1.readFile(aPath, 'utf-8', function (errRead, data) {
            // check if the data changes
            if (!errRead && (data === aData)) {
                observer.complete();
            }
            else {
                // override
                graceful_fs_1.writeFile(aPath, aData, function (errWrite) {
                    if (errWrite) {
                        observer.error(errWrite);
                    }
                    else {
                        observer.next(aPath);
                        observer.complete();
                    }
                });
            }
        });
    });
}
exports.writeFileIfChanged = _writeFileIfChanged;
function _writeFile(aPath, aData) {
    return rxjs_1.Observable.create(function (observer) {
        graceful_fs_1.writeFile(aPath, aData, function (err) {
            if (err) {
                observer.error(err);
            }
            else {
                observer.next(aPath);
                observer.complete();
            }
        });
    });
}
exports.writeFile = _writeFile;
function _readFile(aPath) {
    return rxjs_1.Observable.create(function (observer) {
        //       const t1 = new Date();
        setTimeout(function () {
            graceful_fs_1.readFile(aPath, 'utf-8', function (err, data) {
                //            const t2 = new Date();
                //            console.log('read', aPath, t2.getTime() - t1.getTime());
                if (err) {
                    observer.error(err);
                }
                else {
                    observer.next(data);
                    observer.complete();
                }
            });
        }, 100);
    });
    //    .tap(noop, undefined, () => console.log('readFile', aPath));
}
exports.readFile = _readFile;
function _readFileBuffer(aPath) {
    return rxjs_1.Observable.create(function (observer) {
        graceful_fs_1.readFile(aPath, function (err, data) {
            if (err) {
                observer.error(err);
            }
            else {
                observer.next(data);
                observer.complete();
            }
        });
    });
}
// delete
var _rxRemove = rxjs_1.bindNodeCallback(fs_extra_1.remove);
var _rxDeleteFile = function (aPath) { return _rxRemove(aPath).pipe(operators_1.mapTo(aPath)); };
exports.rxDeleteFile = _rxDeleteFile;
// mkdir
var _rxEnsureDir = rxjs_1.bindNodeCallback(fs_extra_1.ensureDir);
var _rxMkdirp = function (aPath) { return _rxEnsureDir(aPath).pipe(operators_1.mapTo(aPath)); };
exports.rxMkdirp = _rxMkdirp;
// stats
var _rxStats = rxjs_1.bindNodeCallback(graceful_fs_1.stat);
exports.rxStats = _rxStats;
// copy
var _rxCopy = rxjs_1.bindNodeCallback(fs_extra_1.copy);
var _rxCopyFile = function (aSrc, aDst, aOverride) {
    return _rxCopy(aSrc, aDst, { overwrite: !!aOverride, preserveTimestamps: true }).pipe(operators_1.mapTo(aDst));
};
exports.rxCopyFile = _rxCopyFile;
function rxCopyFileStream(aSrc, aDst) {
    return rxjs_1.Observable.create(function (aObserver) {
        var onError = function (err) { return aObserver.error(err); };
        var onOk = function () {
            aObserver.next(aDst);
            aObserver.complete();
        };
        var rd = graceful_fs_1.createReadStream(aSrc);
        rd.on('error', onError);
        var wr = graceful_fs_1.createWriteStream(aDst);
        wr.on('error', onError);
        wr.on('close', onOk);
        rd.pipe(wr);
    });
}
exports.rxCopyFileStream = rxCopyFileStream;
var _rxReaddir = rxjs_1.bindNodeCallback(graceful_fs_1.readdir);
exports.rxReaddir = _rxReaddir;
function _validateNotExists(aDir) {
    return _rxStats(aDir).pipe(operators_1.catchError(function (err) { return rxjs_1.of(err); }), operators_1.mergeMap(function (data) { return (data instanceof Error) ? rxjs_1.of(aDir)
        : rxjs_1.throwError("Directory [" + aDir + "] already exists. Use --override to overwrite it."); }));
}
exports.rxValidateNotExists = _validateNotExists;
function _rxTempDir() {
    return rxjs_1.Observable.create(function (o) {
        // dispatch
        tmp_1.dir(function (tmpError, tmpPath) {
            // test for error
            if (tmpError) {
                o.error(new verror_1.VError(tmpError, 'Unable to create temporary directory.'));
            }
            else {
                o.next(tmpPath);
                o.complete();
            }
        });
    });
}
exports.rxTempDir = _rxTempDir;
