"use strict";
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var path_1 = require("path");
var operators_1 = require("rxjs/operators");
var rx_file_1 = require("./rx.file");
var rx_walk_1 = require("./rx.walk");
var VAR_LAYOUTS = 'LAYOUTS';
var SPREAD_LAYOUTS = '...' + VAR_LAYOUTS;
var PROP_DECLARATIONS = 'declarations';
var PROP_ENTRY_COMPONENTS = 'entryComponents';
function _isEmpty(aValue) {
    return !aValue || (aValue.trim().length === 0);
}
function _addToArray(aValue, aPrefix, aInfix, aSuffix) {
    // check
    if (_isEmpty(aInfix)) {
        return "" + aPrefix + aInfix + SPREAD_LAYOUTS + aSuffix;
    }
    // check
    if (aInfix.includes(SPREAD_LAYOUTS)) {
        return aValue;
    }
    // append
    return "" + aPrefix + aInfix + ", " + SPREAD_LAYOUTS + aSuffix;
}
function _replaceModule(aValue, aPrefix, aInfix, aSuffix) {
    // declarations
    var declEx = /(declarations:\s+\[)([^\]]*)(\])/g;
    var infix = aInfix.replace(declEx, _addToArray);
    // check if there exist entry components
    if (!declEx.test(infix)) {
        // insert the declaration
        var sep = _isEmpty(infix) ? '' : ',';
        infix = "\ndeclarations: [" + SPREAD_LAYOUTS + "]" + sep + infix;
    }
    // entry components
    var entryEx = /(entryComponents:\s+\[)([^\]]*)(\])/g;
    infix = infix.replace(entryEx, _addToArray);
    // check if there exist entry components
    if (!entryEx.test(infix)) {
        // insert the declaration
        var sep = _isEmpty(infix) ? '' : ',';
        infix = "\nentryComponents: [" + SPREAD_LAYOUTS + "]" + sep + infix;
    }
    return "" + aPrefix + infix + aSuffix;
}
function _addLayoutViaRegexp(aLayoutExports, aModuleFile) {
    // get the module directory
    var moduleDir = path_1.resolve(aModuleFile, '..');
    // load the file
    return rx_walk_1.readFile(aModuleFile).pipe(operators_1.mergeMap(function (module) {
        // the modified file
        var mod = module;
        // look for the layouts import
        var layoutImport = /import\s+{\s*LAYOUTS\s*}/g;
        if (!layoutImport.test(mod)) {
            // get the relative name
            var parsed = path_1.parse(path_1.relative(moduleDir, aLayoutExports));
            // relative path
            var relPath = lodash_1.replace(path_1.join(parsed.dir, parsed.name), /\\/g, '/');
            // insert the import
            mod =
                "import { LAYOUTS } from './" + relPath + "';\n" + mod;
        }
        // find the module
        var ngModule = /(@NgModule\s*\(\s*{)((?:.|\n|\r)*)(}\)(?:.|\n|\r)*export\s+class)/g;
        mod = mod.replace(ngModule, _replaceModule);
        // save the module
        return rx_file_1.writeTextFileSafe(aModuleFile, mod, true);
    }), operators_1.map(function () { return aModuleFile; }));
}
exports.addLayoutToModule = _addLayoutViaRegexp;
